#!/usr/bin/env python
# -*- coding: UTF-8 -*-

import click, os, time, fnmatch, json
import re

import sass, jsmin, coffeescript
from watchdog.observers import Observer
from watchdog.events import FileSystemEventHandler
from jsonschema import validate, exceptions

default = {
    "jobs" : [
        {
            "type"   : "javascript",
            "watch"  : ["*.js"]
        },
        {
            "type"   : "sass",
            "watch"  : ["*.scss","*.sass"]
        },
        {
            "type"   : "coffeescript",
            "watch"  : ["*.cs"]
        }
    ]  
}

job_schema = {
    "type" : "object",
    "properties" : {
        "type" : {"type" : "string"},
        "files" : {"type" : "array"},
        "watch" : {"type" : "array"},
    },
    "required" : ["type","watch"],
}

config_schema = {
    "type" : "object",
    "properties" : {
        "jobs" : { "type" : "array", "items" : job_schema }
    }
}



class Config(object):
    def __init__(self):
        self.config = None


pass_config = click.make_pass_decorator(Config, ensure=True)

@click.group()
@pass_config
def cli(config):

    if os.path.exists('./jidoka.json'):
        with open('./jidoka.json') as f:
            try:
                config.config = json.loads(f.read())
            except ValueError as e:
                print(click.style('Check your Config: %s' % e.message, fg='red'))
                exit()

        try:
            validate(config.config, config_schema)
        except exceptions.ValidationError as e:
            print(click.style('jidoka.json %s' % e.message, fg='red'))
            exit()
    else:
        config.config = default

    pass

 
class MyHandler(FileSystemEventHandler):
    def __init__(self, config, watch_list):
        self.config = config
        self.watch_list = watch_list
    def on_any_event(self, event):
        if event.event_type in ['modified','created']:
            for i in self.watch_list:
                if fnmatch.fnmatch(event.src_path, i) and '.min.' not in event.src_path:
                    print(click.style('Change Detected',bold=True))
                    for j in self.watch_list[i]:
                        do_job(event.src_path, self.config['jobs'][j])


def do_job(fpath, job):
    """Simple Build Tool for Webdeveloper"""
    if 'files' not in job:
        job['files'] = [fpath] 

    if job['type'] in ['javascript']:
        for f in job['files']:
            print click.style('...', fg='green')+'%s' % f
            with open(f) as js_file:
                output = jsmin.jsmin(js_file.read())
            import re
            f = re.sub(r"\.(?=[^.]*$)", r".min.", f) 
            with open(f, 'w') as fh:
                fh.write(output)

    if job['type'] in ['sass', 'scss']:
        for f in job['files']:
            print click.style('...', fg='green')+'%s' % f
            output = sass.compile(filename=f, output_style='compressed')
            import re
            f = re.sub(r"\.scss|\.sass", r".css", f)            
            with open(f, 'w') as fh:
                fh.write(output)

    if job['type'] in ['coffeescript']:
        for f in job['files']:
            print click.style('...', fg='green')+'%s' % f
            with open(f) as js_file:
                output = coffeescript.compile(js_file.read())
            import re
            f = re.sub(r"cs", r"js", f)            
            with open(f, 'w') as fh:
                fh.write(output)


@cli.command()
@pass_config
def watch(config):
    """Simple Build Tool for Webdeveloper"""

    watch_list = {}
    for i in config.config['jobs']:
        if 'watch' in i: 
            for f in i['watch']:
                if f in watch_list:
                    if config.config['jobs'].index(i) not in watch_list[f]:
                        watch_list[f].append( config.config['jobs'].index(i) )
                else:
                    watch_list[f] = [config.config['jobs'].index(i)]
    
    event_handler = MyHandler(config.config, watch_list)
    observer = Observer()
    observer.schedule(event_handler, path='.', recursive=True)
    observer.start()

    try:
        while True:
            time.sleep(1)
    except KeyboardInterrupt:
        observer.stop()
    observer.join()


@cli.command()
@click.option('--bump', 
    type=click.Choice(['build','patch', 'minor','major']), 
    help='Incrase Patch Number'
    )

def version(bump):
    """Simple Build Tool for Webdeveloper"""
 
    if os.path.exists('version.txt'):
        with open('version.txt', "r") as f:
            v = f.readlines()[1]
    else:
        v = '0.0.0.0'

    major_nr, minor_nr, patch_nr, build_nr =  v.split('.')

    if bump == 'build':
        build_nr = int(time.time())
    elif bump == 'patch':
        patch_nr, build_nr = [int(patch_nr)+1, 0]
    elif bump == 'minor':
        minor_nr, patch_nr, build_nr = [int(minor_nr)+1,0,0]
    elif bump == 'major':
        major_nr, minor_nr, patch_nr, build_nr = [int(major_nr)+1,0,0,0]
    else:
        click.echo('current version is %s.%s.%s.%s' % (major_nr, minor_nr, patch_nr, build_nr))
        return

    with open('version.txt', "w") as f:
        f.write('# The version number is autogenerated. Do not change this file by your own!\n%s.%s.%s.%s\n' % (major_nr, minor_nr, patch_nr, build_nr))

    click.echo('bump version to %s.%s.%s.%s' % (major_nr, minor_nr, patch_nr, build_nr))


if __name__ == '__main__':
    cli()
